# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
=begin 
    
    Copyright:	© 2006, Remi Goux
    
    Name:       rename_layers.rb
    Version:    0.1b

    Install:    Drop into your Plugins directory. Relaunch SU.
    
    Usage:      Choose 'Plugins/Rename Layers...'
                Select either 'All Layers' or 'Only Visible Layers'.
                Enter a format string in the 'With Format' field.
                Click OK or press Return(Enter)
                Choose 'Edit/Undo Rename Layers...' if things went wrong
                
    Format string:  The format string is a pure Ruby quoted string without the end quotes
                    Pre-defined variables are made available:
                        - name: the current layer name
                        - index: the current layer index, starting with 0
                        - number: the current layer number, starting with 1
                        - total: the number of layers in the model
                        - serial: the renamed layers count, starting with 1
                        - status: either 'visible' or 'hidden'
                        - model: the model itself
                    The transformed name is passed to unique_name and may be modified
                    The default format string does nothing:
                        #{name} => Layer4
                    Some examples:
                        Name        Format          New name
                        ---------------------------------------------------------------------
                        Layer4 => #{name.upcase}                        => LAYER4
                        Layer4 => #{name[0..-2]} number #{index}        => Layer number 4
                        Layer4 => DXF_#{name.upcase}.imported           => DXF_LAYER4.imported
                        Layer4 => Foo[#{number}|#{total}]   		=> Foo[5 out 8]
                        Layer4 => #{name}.#{status}                     => Layer4.visible
                        Layer4 => #{name} from #{model.title}           => Layer4 from My Model
                        Layer4 => Calque numero #{index}                => Calque numero 4
                        Layer4 => Layer number #{'%03d' % index}        => Layer number 004
                    
                    Sketchup may assign a unique name, though:
                        Layer4 => Foo => Foo3

    Known Issues:   Layer0 cannot be renamed
                    Format syntax errors are not properly trapped 
    
    Features:   Preserve settings between sessions
    			Menu item disabled when the model has only one layer
    			Undo/Redo
    
    History:    06-30-2006  0.1b    Proposal
    
    todo:		preflight format string and trap syntax errors
    
=end

class LayerRenamer

    @@vlOnly  = Sketchup.read_default(self.name,'vlOnly',false)
    @@format = Sketchup.read_default(self.name,'format',['#{name}'].pack('m')).unpack('m').first
    
    def self.get_format withTitle
    
        itemAllLayers = 'All Layers'
        itemVisibleLayers = 'Only Visible Layers'
        
        return false unless result = UI.inputbox(
            ['Rename:','With Format:'],
            [@@vlOnly ? itemVisibleLayers : itemAllLayers, @@format], 
            [[itemAllLayers, itemVisibleLayers] * '|', nil],
            withTitle)
        @@vlOnly = result[0] == itemVisibleLayers
        @@format = result[1]
        
        Sketchup.write_default self.name, 'vlOnly', @@vlOnly
        Sketchup.write_default self.name,'format',[@@format].pack('m')
        
        true
    end
    
    def self.action model
        index = 0
        number = 1
        serial = 0
        total = model.layers.length
        model.layers.to_a[1..-1].each do |l|
            index += 1
            number += 1
            name = l.name
            if @@vlOnly and !l.visible?
                puts "Ignoring invisible layer '#{name}'"
                next
            end
            serial += 1
            status = l.visible?() ? 'visible' : 'hidden'
            newName = eval('"'+@@format+'"')
            newName = model.layers.unique_name(newName) unless newName.downcase == name.downcase
            puts "Renaming layer '#{name}' to '#{newName}'"
            l.name = newName
        end
        case serial
        when 0
            Sketchup.status_text = "#{self.name}: no layer was renamed."
        when 1
            Sketchup.status_text = "#{self.name}: 1 layer was renamed. Use Undo to reverse."
        else
            Sketchup.status_text = "#{self.name}: #{serial} layers were renamed. Use Undo to reverse."
        end
    end
    
end # class LayerRenamer

class LayerRenamerController

    def self.label
        'Rename Layers...'
    end
    
    def self.build_menus
        layerRenameCommand = UI::Command.new(label) do
            if LayerRenamer.get_format label
                begin
                    Sketchup.active_model.start_operation label
                    LayerRenamer.action Sketchup.active_model
                    Sketchup.active_model.commit_operation
                rescue
                    Sketchup.active_model.abort_operation
                end
            end
        end.set_validation_proc do
            Sketchup.active_model.layers.length > 1 ? MF_ENABLED : MF_GRAYED
        end
        UI.menu('Plugins').add_item layerRenameCommand
        @@menus_built = true
    end
    
    build_menus unless @@menus_built ||= false
    
end # class LayerRenamerController